@@ -1,8 +1,10 @@
+from fpql.termstructure import qlFlatForward, qlPiecewise
 import QuantLib as ql 
 from .pymodels import *
 from .ql_utils import *
 from .ql_conventions import *
 from .ql_enums import *
+from . import pricingmodels as pm
 
 
 def get_qlfixedratebond(bond_info: FixedRateBond, setting: BondSetting,
@@ -214,3 +216,78 @@ def get_qlcallablebond(bond_info: FixedRateBond,
                 call_schedule)
     return callbond
 
+
+def callbond_stats(value_date: str,
+                    call_class: CallableBondParam, 
+                    ql_calendar,
+                    clean_price: float,
+                    engine: ql.QuantLib.TreeCallableFixedRateBondEngine,
+                    curve: Optional[ql.QuantLib.YieldTermStructureHandle] = None):
+    
+    vdate = datestr_to_qldate(value_date)
+    ql_callbond = qlCallableBond_from_Class(call_class, ql_calendar)
+    if not curve:
+        if call_class.use_flat_curve:
+            if call_class.flat_curve:
+                ffwd = qlFlatForward(call_class.flat_curve)
+                curve_handle = ql.YieldTermStructureHandle(ffwd)
+        else:
+            pwc = qlPiecewise(call_class.piecewise_curve, ql_calendar)
+            curve_handle = ql.YieldTermStructureHandle(pwc)
+    else:
+        curve_handle = curve
+
+    ql_callbond.setPricingEngine(engine)
+    call_risks = {}
+    
+    oas = ql_callbond.OAS(clean_price, 
+                        curve_handle,
+                        ql_day_count[call_class.setting.day_count],
+                        ql.Compounded,
+                        ql_frequency[call_class.setting.frequency],
+                        vdate) * 100
+    call_risks["OAS"] =  oas
+    bond_yield  = ql_callbond.bondYield(clean_price,
+                        ql_day_count[call_class.setting.day_count],
+                        ql.Compounded,
+                        ql_frequency[call_class.setting.frequency],
+                        vdate) * 100
+    call_risks["Yield"] = bond_yield
+
+    e_convex = ql_callbond.effectiveConvexity(oas/100, 
+                                curve_handle,
+                                ql_day_count[call_class.setting.day_count],
+                                ql.Compounded,
+                                ql_frequency[call_class.setting.frequency])
+    call_risks["Effective Convexity"] = e_convex
+
+    e_duration = ql_callbond.effectiveConvexity(oas/100, 
+                                curve_handle,
+                                ql_day_count[call_class.setting.day_count],
+                                ql.Compounded,
+                                ql_frequency[call_class.setting.frequency])
+    call_risks["Effective Duration"] = e_duration
+    
+    #Calculate Yield To Call
+
+    return call_risks
+
+
+def qlCallableBond_from_Class(call_class: CallableBondParam,
+                            qlcalendar):
+    qlschedule = qlSchedule_from_Model(call_class.info,
+                                        call_class.setting, qlcalendar) 
+    structure = structure_from_qlSchedule(call_class.info,
+                                            call_class.setting,
+                                            qlschedule)
+    call_class.structure = structure
+    qlcall_schedule = qlCallabilitySchedule_from_class(call_class.call_structure)
+    qlcallbond = get_qlcallablebond(call_class.info,
+                                    call_class.setting, 
+                                    qlschedule, 
+                                    [call_class.info.coupon/100], 
+                                    qlcall_schedule )
+    
+    return qlcallbond
+
+        
\ No newline at end of file
