@@ -1,8 +1,8 @@
 import QuantLib as ql
 from typing import List
-from pymodels import *
-from ql_utils import *
-from ql_conventions import *
+from .pymodels import *
+from .ql_utils import *
+from .ql_conventions import *
 
 
 def deporate_2_depohelpers(depo_setting: DepoSetting, rates: List[Rate],  calendar=None ):
