@@ -1,3 +1,5 @@
+import datetime
+
 from .ql_enums import *
 from .ql_conventions import *
 
@@ -6,6 +8,7 @@ from typing import List, Optional
 from numpy import datetime64 as dt64
 
 
+
 class Rate(BaseModel):
     tenor: str
     rate: float
@@ -285,3 +288,65 @@ class CallabilityPrice(BaseModel):
     date: str
     option_type: OptionType = OptionType.calloption
 
+
+class FlatForwardCurve(BaseModel):
+    date: str
+    rate: float
+    day_count: DayCount
+    frequency: Frequency
+    compounding: Optional[Compounding] = "Compounded" 
+
+
+class PiecewiseCurve(BaseModel):
+    value_date: str
+    depo_setting: DepoSetting
+    depo_rates: List[Rate]
+    par_setting: BondSetting
+    par_rates: List[Rate]
+    method: PiecewiseMethods = PiecewiseMethods.logcubicdiscount
+
+
+class CallableRisks(BaseModel):
+    oas: Optional[float] = None
+    bond_yield: Optional[float] = None
+    ytc: Optional[float] = None
+    eff_convexity: Optional[float] = None
+    eff_duration: Optional[float] = None
+
+
+class CallableBondParam(BaseModel):
+    flat_curve: Optional[FlatForwardCurve] = None
+    piecewise_curve:Optional[PiecewiseCurve] = None
+    info: FixedRateBond
+    setting: BondSetting
+    structure: List[Structure] = []
+    call_structure: List[CallabilityPrice] = []
+    risks: Optional[CallableRisks] = None
+    irmodel: Optional[ShortRateModel] = ShortRateModel.hullwhite
+    use_flat_curve: bool = True
+
+
+class SRMVasicek(BaseModel):
+    r0: float = 0.05
+    a: float = 0.1
+    b: float = 0.05
+    sigma: float = 0.1
+    lamda: float = 0.0
+
+
+class SRMBlackKaransinski(BaseModel):
+    a: float = 0.1
+    sigma: float = 0.1
+
+
+class SRMHullWhite(BaseModel):
+    a: float = 0.1
+    sigma: float = 0.01
+
+
+class SRMG2(BaseModel):
+    a: float = 0.1
+    b: float = 0.1
+    sigma: float = 0.01
+    eta: float = 0.01
+    rho: float = -0.75
